@@ -979,6 +979,16 @@ class IfcfgNetConfig(os_net_config.NetConfig):
         logger.debug('ovs dpdk port data: %s' % data)
         self.interface_data[ovs_dpdk_port.name] = data
 
+        """Add an extra ifcfg entry for mellanox NICs,
+        for the interface which is a member of ovs_dpdk_port
+        with NM and DHCP disabled
+        """
+        if utils.is_mellanox_interface(ifname):
+            self.nm_controlled = False
+            self.use_dhcp = False
+            dpdk_if_name = objects.Interface(ifname)
+            self.add_interface(dpdk_if_name)
+
     def add_ovs_dpdk_bond(self, ovs_dpdk_bond):
         """Add an OvsDPDKBond object to the net config object.
 
@@ -1003,6 +1013,18 @@ class IfcfgNetConfig(os_net_config.NetConfig):
         if ovs_dpdk_bond.rules:
             self._add_rules(ovs_dpdk_bond.name, ovs_dpdk_bond.rules)
 
+        """Add an extra ifcfg entry for mellanox NICs,
+        for all interfaces which is a member of ovs_dpdk_bond
+        with NM and DHCP disabled
+        """
+        for dpdk_port in ovs_dpdk_bond.members:
+            ifname = dpdk_port.members[0].name
+            if utils.is_mellanox_interface(ifname):
+                self.nm_controlled = False
+                self.use_dhcp = False
+                dpdk_if_name = objects.Interface(ifname)
+                self.add_interface(dpdk_if_name)
+
     def add_sriov_pf(self, sriov_pf):
         """Add a SriovPF object to the net config object
 
@@ -1013,7 +1035,8 @@ class IfcfgNetConfig(os_net_config.NetConfig):
         logger.debug('sriov pf data: %s' % data)
         utils.update_sriov_pf_map(sriov_pf.name, sriov_pf.numvfs,
                                   self.noop, promisc=sriov_pf.promisc,
-                                  link_mode=sriov_pf.link_mode)
+                                  link_mode=sriov_pf.link_mode,
+                                  steering_mode=sriov_pf.steering_mode)
         self.interface_data[sriov_pf.name] = data
         if sriov_pf.routes:
             self._add_routes(sriov_pf.name, sriov_pf.routes)
@@ -1069,13 +1092,11 @@ class IfcfgNetConfig(os_net_config.NetConfig):
         """
         logger.info('adding contrail_vrouter interface: %s'
                     % contrail_vrouter.name)
-        # Contrail vrouter will have the only member (of type interface,
-        # vlan or linux_bond)
-        ifname = contrail_vrouter.members[0].name
+        ifnames = ",".join([m.name for m in contrail_vrouter.members])
         data = self._add_common(contrail_vrouter)
         data += "DEVICETYPE=vhost\n"
         data += "TYPE=kernel_mode\n"
-        data += "BIND_INT=%s\n" % ifname
+        data += "BIND_INT=%s\n" % ifnames
         logger.debug('contrail data: %s' % data)
         self.interface_data[contrail_vrouter.name] = data
         if contrail_vrouter.routes:
@@ -1114,6 +1135,24 @@ class IfcfgNetConfig(os_net_config.NetConfig):
             self._add_rules(contrail_vrouter_dpdk.name,
                             contrail_vrouter_dpdk.rules)
 
+    def add_linux_tap(self, linux_tap):
+        """Add a LinuxTap object to the net config object
+
+        :param linux_tap:
+           The LinuxTap object to add
+        """
+        logger.info('adding Linux TAP interface: %s'
+                    % linux_tap.name)
+        data = self._add_common(linux_tap)
+        data += "TYPE=Tap\n"
+        self.interface_data[linux_tap.name] = data
+        if linux_tap.routes:
+            self._add_routes(linux_tap.name,
+                             linux_tap.routes)
+        if linux_tap.rules:
+            self._add_rules(linux_tap.name,
+                            linux_tap.rules)
+
     def generate_ivs_config(self, ivs_uplinks, ivs_interfaces):
         """Generate configuration content for ivs."""
 
