@@ -1019,9 +1019,9 @@ class LinuxBond(_BaseOpts):
                         % (iface.device, iface.vfid))
             iface.trust = 'on'
         if iface.spoofcheck is None:
-            logger.info("Spoofcheck is not set for VF %s:%d, defaulting to on"
+            logger.info("Spoofcheck is not set for VF %s:%d, defaulting to off"
                         % (iface.device, iface.vfid))
-            iface.spoofcheck = 'on'
+            iface.spoofcheck = 'off'
         if iface.promisc is None:
             logger.info("Promisc is not set for VF %s:%d, defaulting to off"
                         % (iface.device, iface.vfid))
@@ -1033,7 +1033,8 @@ class LinuxBond(_BaseOpts):
                                   macaddr=iface.macaddr, promisc=iface.promisc,
                                   pci_address=iface.pci_address,
                                   min_tx_rate=iface.min_tx_rate,
-                                  max_tx_rate=iface.max_tx_rate)
+                                  max_tx_rate=iface.max_tx_rate,
+                                  driver=None)
 
     @staticmethod
     def from_json(json):
@@ -1119,7 +1120,8 @@ class OvsBond(_BaseOpts):
                                   trust=iface.trust, state=iface.state,
                                   macaddr=iface.macaddr, promisc=iface.promisc,
                                   min_tx_rate=iface.min_tx_rate,
-                                  max_tx_rate=iface.max_tx_rate)
+                                  max_tx_rate=iface.max_tx_rate,
+                                  driver=None)
 
     @staticmethod
     def from_json(json):
@@ -1328,7 +1330,7 @@ class OvsDpdkPort(_BaseOpts):
         self.rx_queue = rx_queue
 
     @staticmethod
-    def update_vf_config(iface):
+    def update_vf_config(iface, driver=None):
         if iface.trust is None:
             logger.info("Trust is not set for VF %s:%d, defaulting to on"
                         % (iface.device, iface.vfid))
@@ -1340,6 +1342,7 @@ class OvsDpdkPort(_BaseOpts):
         if iface.promisc is not None:
             logger.warning("Promisc can't be changed for ovs_dpdk_port")
             iface.promisc = None
+        logger.info("Overriding the default driver for DPDK")
         utils.update_sriov_vf_map(iface.device, iface.vfid, iface.name,
                                   vlan_id=iface.vlan_id, qos=iface.qos,
                                   spoofcheck=iface.spoofcheck,
@@ -1347,7 +1350,8 @@ class OvsDpdkPort(_BaseOpts):
                                   macaddr=iface.macaddr, promisc=iface.promisc,
                                   pci_address=iface.pci_address,
                                   min_tx_rate=iface.min_tx_rate,
-                                  max_tx_rate=iface.max_tx_rate)
+                                  max_tx_rate=iface.max_tx_rate,
+                                  driver=driver)
 
     @staticmethod
     def from_json(json):
@@ -1379,7 +1383,7 @@ class OvsDpdkPort(_BaseOpts):
                         # be set in the interface part of DPDK Port
                         members.append(iface)
                     elif isinstance(iface, SriovVF):
-                        OvsDpdkPort.update_vf_config(iface)
+                        OvsDpdkPort.update_vf_config(iface, driver)
                         members.append(iface)
                     else:
                         msg = 'Unsupported OVS DPDK Port member type'
@@ -1453,6 +1457,8 @@ class SriovVF(_BaseOpts):
         self.macaddr = macaddr
         self.promisc = promisc
         self.pci_address = pci_address
+        self.driver = None
+
         utils.update_sriov_vf_map(device, self.vfid, name,
                                   vlan_id=self.vlan_id,
                                   qos=self.qos,
@@ -1463,7 +1469,8 @@ class SriovVF(_BaseOpts):
                                   promisc=promisc,
                                   pci_address=pci_address,
                                   min_tx_rate=min_tx_rate,
-                                  max_tx_rate=max_tx_rate)
+                                  max_tx_rate=max_tx_rate,
+                                  driver=self.driver)
 
     @staticmethod
     def get_on_off(config):
@@ -1515,7 +1522,8 @@ class SriovPF(_BaseOpts):
                  primary=False, nic_mapping=None, persist_mapping=False,
                  defroute=True, dhclient_args=None, dns_servers=None,
                  nm_controlled=False, onboot=True, domain=None, members=None,
-                 promisc=None, link_mode='legacy', ethtool_opts=None):
+                 promisc=None, link_mode='legacy', ethtool_opts=None,
+                 steering_mode=None):
         addresses = addresses or []
         routes = routes or []
         rules = rules or []
@@ -1534,6 +1542,7 @@ class SriovPF(_BaseOpts):
         self.promisc = promisc
         self.link_mode = link_mode
         self.ethtool_opts = ethtool_opts
+        self.steering_mode = steering_mode
 
     @staticmethod
     def get_on_off(config):
@@ -1553,12 +1562,17 @@ class SriovPF(_BaseOpts):
         promisc = SriovPF.get_on_off(promisc)
         link_mode = json.get('link_mode', 'legacy')
         ethtool_opts = json.get('ethtool_opts', None)
+        steering_mode = json.get('steering_mode')
+        if steering_mode is not None and steering_mode not in ['smfs', 'dmfs']:
+            msg = 'Expecting steering_mode to match smfs/dmfs'
+            raise InvalidConfigException(msg)
         if link_mode not in ['legacy', 'switchdev']:
             msg = 'Expecting link_mode to match legacy/switchdev'
             raise InvalidConfigException(msg)
         opts = _BaseOpts.base_opts_from_json(json)
         return SriovPF(name, numvfs, *opts, promisc=promisc,
-                       link_mode=link_mode, ethtool_opts=ethtool_opts)
+                       link_mode=link_mode, ethtool_opts=ethtool_opts,
+                       steering_mode=steering_mode)
 
 
 class OvsDpdkBond(_BaseOpts):
