@@ -689,7 +689,8 @@ class TestBridge(base.TestCase):
                      'vlan_id': 111, 'qos': 1,
                      'min_tx_rate': 0, 'max_tx_rate': 0,
                      'spoofcheck': 'off', 'trust': 'on',
-                     'pci_address': '0000:79:10.2'
+                     'pci_address': '0000:79:10.2',
+                     'driver': 'vfio-pci'
                      }]
 
         def test_get_vf_devname(device, vfid):
@@ -745,7 +746,8 @@ class TestBridge(base.TestCase):
                      'vlan_id': 111, 'qos': 1,
                      'min_tx_rate': 100, 'max_tx_rate': 500,
                      'spoofcheck': 'off', 'trust': 'off',
-                     'pci_address': '0000:79:10.2'
+                     'pci_address': '0000:79:10.2',
+                     'driver': 'vfio-pci'
                      }]
 
         def test_get_vf_devname(device, vfid):
@@ -1257,14 +1259,14 @@ class TestLinuxBond(base.TestCase):
                      'vlan_id': 111, 'qos': 1,
                      'min_tx_rate': 0, 'max_tx_rate': 10,
                      'pci_address': '0000:79:10.1',
-                     'spoofcheck': 'on', 'trust': 'on',
+                     'spoofcheck': 'off', 'trust': 'on',
                      'promisc': 'off'},
                     {'device_type': 'vf', 'name': 'em2_1',
                      'device': {'name': 'em2', 'vfid': 1},
                      'vlan_id': 111, 'qos': 1,
                      'min_tx_rate': 0, 'max_tx_rate': 10,
                      'pci_address': '0000:79:10.2',
-                     'spoofcheck': 'on', 'trust': 'on',
+                     'spoofcheck': 'off', 'trust': 'on',
                      'promisc': 'off'}]
 
         def test_get_vf_devname(device, vfid):
@@ -1984,6 +1986,52 @@ class TestSriovPF(base.TestCase):
         expected = 'Expecting link_mode to match legacy/switchdev'
         self.assertIn(expected, six.text_type(err))
 
+    def test_from_json_steering_mode_dmfs(self):
+        data = '{"type": "sriov_pf", "name": "p6p1", "numvfs": 8,' \
+               '"use_dhcp": false, "promisc": false, "link_mode":' \
+               '"switchdev", "steering_mode": "dmfs"}'
+        pf = objects.object_from_json(json.loads(data))
+        self.assertEqual("p6p1", pf.name)
+        self.assertEqual(8, pf.numvfs)
+        self.assertEqual("off", pf.promisc)
+        self.assertFalse(pf.use_dhcp)
+        self.assertEqual("switchdev", pf.link_mode)
+        self.assertEqual("dmfs", pf.steering_mode)
+
+    def test_from_json_steering_mode_smfs(self):
+        data = '{"type": "sriov_pf", "name": "p6p1", "numvfs": 8,' \
+               '"use_dhcp": false, "promisc": false, "link_mode":' \
+               '"switchdev", "steering_mode": "smfs"}'
+        pf = objects.object_from_json(json.loads(data))
+        self.assertEqual("p6p1", pf.name)
+        self.assertEqual(8, pf.numvfs)
+        self.assertEqual("off", pf.promisc)
+        self.assertFalse(pf.use_dhcp)
+        self.assertEqual("switchdev", pf.link_mode)
+        self.assertEqual("smfs", pf.steering_mode)
+
+    def test_from_json_steering_mode_none(self):
+        data = '{"type": "sriov_pf", "name": "p6p1", "numvfs": 8,' \
+               '"use_dhcp": false, "promisc": false, "link_mode":' \
+               '"switchdev"}'
+        pf = objects.object_from_json(json.loads(data))
+        self.assertEqual("p6p1", pf.name)
+        self.assertEqual(8, pf.numvfs)
+        self.assertEqual("off", pf.promisc)
+        self.assertFalse(pf.use_dhcp)
+        self.assertEqual("switchdev", pf.link_mode)
+        self.assertIsNone(pf.steering_mode)
+
+    def test_from_json_steering_mode_invalid(self):
+        data = '{"type": "sriov_pf", "name": "p6p1", "numvfs": 8,' \
+               '"use_dhcp": false, "promisc": false, "link_mode":' \
+               '"switchdev", "steering_mode": "dmfss"}'
+        err = self.assertRaises(objects.InvalidConfigException,
+                                objects.object_from_json,
+                                json.loads(data))
+        expected = 'Expecting steering_mode to match smfs/dmfs'
+        self.assertIn(expected, str(err))
+
     def test_from_json_ethtool_opts(self):
         data = '{"type": "sriov_pf", "name": "em1", "numvfs": 16, ' \
                '"use_dhcp": false, "promisc": false, ' \
@@ -2443,3 +2491,18 @@ class TestOvsRequiredObjects(base.TestCase):
         expected = 'OvsDpdkPort cannot be created as OpenvSwitch is not ' \
                    'installed.'
         self.assertIn(expected, six.text_type(err))
+
+
+class TestLinuxTap(base.TestCase):
+
+    def test_linux_tap_from_json(self):
+        data = """{
+"type": "linux_tap",
+"name": "tap0",
+"nm_controlled": false
+}
+"""
+
+        linux_tap = objects.object_from_json(json.loads(data))
+        self.assertEqual("tap0", linux_tap.name)
+        self.assertEqual(False, linux_tap.nm_controlled)
